require "listen"

# Restarts Puma when a file gets modified.
# Only works when RACK_ENV == "development" to prevent accidental usage.
#
# By default it restarts Puma on every file change inside the current directory,
# but can be configured through the following environment variables:
#
# - ENV["PUMA_FILE_RESTART_DIRS"]: comma-separated list of the directories
#   to be monitored. Example: "./app,./tmp"
#
# - ENV["PUMA_FILE_RESTART_IGNORE"]: string representing a Ruby Regexp.
#   Changes in files whose pathname match the Regexp won"t restart Puma.
#   Example: '\.txt\Z' will ignore changes in any file with extension .txt
#   (Notice the single quotes so you don't need to worry about escaping '\').
#
# - ENV["PUMA_FILE_RESTART_ONLY"]: string representing a Ruby Regexp.
#   Changes in files whose pathname don"t match the Regexp won"t restart Puma.
#   Example: '\.txt\Z' will changes in any file with extension .txt
#   (Notice the single quotes so you don't need to worry about escaping '\').

Puma::Plugin.create do
  def start(launcher)
    return unless ENV["RACK_ENV"] == "development"
    in_background do
      listener = Listen.to(*dirs) { |_| launcher.restart }
      listener.only(only) if only
      listener.ignore(ignore) if ignore
      listener.start
      sleep
    end
  end

  def dirs
    if env = ENV["PUMA_FILE_RESTART_DIRS"]
      env.split(",")
    else
      [Dir.pwd]
    end
  end

  def ignore
    if env = ENV["PUMA_FILE_RESTART_IGNORE"]
      Regexp.new(env)
    end
  end

  def only
    if env = ENV["PUMA_FILE_RESTART_ONLY"]
      Regexp.new(env)
    end
  end

  VERSION = "0.0.1"
end
